/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */

package org.bsf.listOfValues;

import org.bsf.commons.ejb.EntityAdapterBean;
import org.bsf.listOfValues.exceptions.NoSuchLovValueException;
import org.bsf.listOfValues.lovValue.DefaultLovValue;
import org.bsf.listOfValues.lovValue.LovValue;
import org.bsf.listOfValues.util.LovDefinitionCache;
import org.bsf.listOfValues.util.LovTypeManager;

import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.ObjectNotFoundException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

/**
 * The LovBean is a BMP EJB that enables the retrieval of a List of Values (LOV).
 * It uses two tables to retrieve the informations needed to build the LOV.<p>
 *
 * Description of the two tables used by this BMP:
 *
 * 1) BSF_LOV         : The definition of the LOV itself (what to retrieve)
 *
 *    => OID          : The oid of the LOV   (ex: 1)
 *    => LOV_NAME     : The name of the LOV, only for information (ex: LOV_COUNTRIES)
 *    => LOV_REQUEST  : The request to execute to retrieve the list of values.
 *                    : The first returned column will define the PK of the
 *                    : LovValue(cf. S_LOV_DEFINITON below).
 *                    : (ex: select oid, short_name, long_name from countries)
 *    => TABLE_NAME   : The name of the Table used in this LOV (could be usefull
 *                    : when we need to notify that the LOV has to be retrieved
 *                    : again since the underlying table has changed). If a LOV
 *                    : is based on more than one table set the "most important"
 *                    : one (or the one that changes the most... it is up to you).
 *
 * 2) BSF_LOV_DEFINITION : The definition of the fields contained in the LovValues
 *                       : returned from the execution of the SQL statement cor-
 *                       : responding to the selected LOV (cf. BSF_LOV).
 *                       : Enables to use the expected data type and to label the
 *                       : fields.
 *
 *    => LOV_OID        : The OID of the LOV (cf. BSF_LOV) for which we define
 *                      : a field.
 *    => FIELD_NAME     : The String used to place and access the column matching
 *                      : the given field_position of the the LOV_REQUEST in the
 *                      : returned LovValue. Keep the name simple...
 *    => FIELD_TYPE     : The type of the field (ie. java.lang.Long, java.lang.String,
 *                      : java.lang.Double, ...).
 *    => FIELD_POSITION : The position of this field in the SQL statement defined
 *                      : in the LOV_REQUEST column of the BSF_LOV table. The first
 *                      : column defined will be used as the PK for the LovValues.
 *
 * @see LovServiceBean
 * @see LovBusinessInterface
 * @see org.bsf.listOfValues.lovValue.LovValue
 *
 *  @ejb.bean            type="BMP"
 *                       name="LOV"
 *                       local-jndi-name="ejb/LOVLocal"
 *                       view-type="local"
 *
 *  @ejb.home            local-extends="javax.ejb.EJBLocalHome"
 *
 *  @ejb.interface       local-extends="javax.ejb.EJBLocalObject, org.bsf.listOfValues.LovBusinessInterface"
 *
 *  @ejb.pk              class="java.lang.Long"
 *                       generate="false"
 *
 *  @ejb.transaction     type="Supports"
 *
 *  @ejb.resource-ref    res-name="jdbc/DBPool"
 *                       res-type="javax.sql.DataSource"
 *                       res-auth="Container"
 *
 *  @jboss.read-only     read-only="true"
 *
 *  @jboss.resource-ref  res-ref-name="jdbc/DBPool"
 *                       resource-name="FwkDataSourceManager"
 *
 *  @jonas.bean          ejb-name="LOV"
 *
 *  @jonas.resource      res-ref-name="jdbc/DBPool"
 *                       jndi-name="jdbc_1"
 */
public class LovBean extends EntityAdapterBean implements LovBusinessInterface {
    // SQL Statements used in this BMP
    private static final String NAME_AND_REQUEST_FOR_OID_STATEMENT
            = "SELECT LOV_NAME, LOV_REQUEST FROM BSF_LOV WHERE OID = ?";

    private static final String METADATA_FOR_OID_STATEMENT
            = "SELECT FIELD_NAME, FIELD_TYPE FROM BSF_LOV_DEFINITION " +
              "WHERE LOV_OID = ? ORDER BY FIELD_POSITION";

    public static final String LOV_OID_FOR_TABLE_NAME_STATEMENT
            = "SELECT OID FROM BSF_LOV WHERE TABLE_NAME = ?";

    /**
     * The JNDI name used to retrieve a connection to the database.
     */
    private final String JNDI_DATABASE_NAME = "DBPool";

    /**
     * Used as a cache to store the results of the executed SQL to prevent the
     * reexecution of the SQL statement each time the same LOV is requested).
     */
    private Vector _sortedLov;

    /**
     * Stores the lovValues in their "mapped" (OID, LovValue) form (a HashMap)
     * to prevent remapping each time the map is requested.
     */
    private HashMap _lov;

    /**
     * Stores the metadata (field name, position...) for this list of values.
     */
    private List _metaData;

    /**
     *  Loads all the values from the DB using the request defined in the
     *  LOV_REQUESTS table. For each row of the resultSet uses the
     *  LOV_DEFINITION_NAME and LOV_DEFINITION_TYPE from table LOV_DEFINITION
     *  to create the lovValue. This method should be called once per LOV
     */
    public void ejbLoad() {
        String lovRequest = null;
        String lovTableName = null;
        ResultSet resultSet = null;
        Statement statement = null;
        Connection connection = null;
        ResultSet resultSetCache = null;
        ResultSet resultSetMetaData = null;
        PreparedStatement preparedStatement = null;

        // We retrieve the OID of the list of values that we want to retrieve
        Long lovRequestID = (Long) ejbContext.getPrimaryKey();

        logGraphBegin( "ejbLoad() with OID " + lovRequestID.toString() );

        try {
            // We retrieve the connextion to the database pool
            connection = getConnection( JNDI_DATABASE_NAME );

            // We don't want to get the SQL Statement from the DB if we have
            // already retrieved it (cf. LovDefinitionCache)... First things first, we
            // check in the LovDefinitionCache (result will be null if no match is found)
            lovRequest = LovDefinitionCache.getInstance().getRequestForLovOID( lovRequestID );

            // Do we need to retrieve the SQL preparedStatement from the DB ?
            // (ie. not in cache, see above)
            if ( lovRequest == null ) {
                // We create a sql preparedStatement using the selected connection
                preparedStatement = connection.prepareStatement( NAME_AND_REQUEST_FOR_OID_STATEMENT );
                preparedStatement.setString( 1, lovRequestID.toString() );
                resultSetCache = preparedStatement.executeQuery();

                if ( resultSetCache.next() == false ) {
                    String msg = "No existing LOV for OID " + lovRequestID;

                    // We log the error (should not happen but in dev or mishandling of the DB...)
                    logError( msg );

                    throw new EJBException( msg );
                }

                // cf. the NAME_AND_REQUEST_FOR_OID_STATEMENT for the parameters order
                lovTableName = resultSetCache.getString( 1 );
                lovRequest = resultSetCache.getString( 2 );

                preparedStatement.close();
            }

            _metaData = LovDefinitionCache.getInstance().getTypesForLovOID( lovRequestID );

            // Next we check for the meta data
            if ( _metaData == null ) {
                preparedStatement = connection.prepareStatement( METADATA_FOR_OID_STATEMENT );
                preparedStatement.setString( 1, lovRequestID.toString() );
                resultSetMetaData = preparedStatement.executeQuery();

                _metaData = new Vector();

                // For each field we retrieve the name and the type
                while ( resultSetMetaData.next() ) {
                    String fieldName = resultSetMetaData.getString( 1 );
                    String fieldType = resultSetMetaData.getString( 2 );
                    LovMetaDataItem metaDataItem = new LovMetaDataItem( fieldName, fieldType );

                    // The infos are ordered by the ORDER BY of the SQL preparedStatement
                    _metaData.add( metaDataItem );
                }
            }

            if ( !LovDefinitionCache.getInstance().isOIDInCache( lovRequestID ) ) {
                // We place the newly acquired informations in the LovDefinitionCache
                LovDefinitionCache.getInstance().addToCache( lovRequestID, lovTableName, lovRequest, _metaData );
            }

            // Then we retrieve the values for the list of values
            logDebug( "Executing : " + lovRequest );

            // We execute the query
            statement = connection.createStatement();
            resultSet = statement.executeQuery( lovRequest );

            // We ensure that the number of columns in the resultset equals the number
            // of fields defined in the metadata (cf. above)
            int numberOfColumns = resultSet.getMetaData().getColumnCount();

            if ( numberOfColumns != _metaData.size() ) {
                // We should have this equallity since the OID field is present
                // even though it is not defined in the S_LOV_DEFINITION table
                String msg = "The lov " + lovRequestID + " is badly parametered (check fields definition... PK must be defined in first position...)";

                logFatal( msg );

                throw new EJBException( msg );
            }

            _lov = new HashMap();
            _sortedLov = new Vector();

            while ( resultSet.next() ) {
                // Remember that the PK of the LovValue MUST be in the first column
                DefaultLovValue lovValue = null;

                // Now we get each field of the lovValue
                for ( int index = 0; index < _metaData.size(); index++ ) {
                    LovMetaDataItem metaDataItem = (LovMetaDataItem) _metaData.get( index );
                    String lovValueFieldType = metaDataItem.getFieldType();
                    String lovValueFieldName = metaDataItem.getFieldName();

                    Object lovValueFieldObject = null;

                    try {
                        // We retrieve the value from the resultset with the corret type
                        lovValueFieldObject = LovTypeManager.getObjectOfType( lovValueFieldType, resultSet, (int) index + 1 );

                        if ( index == 0 ) {
                            // Different handling the PK... We create the LovValue with it
                            lovValue = new DefaultLovValue( lovValueFieldObject );
                        } else {
                            // We add the value to the LovValue
                            lovValue.addField( lovValueFieldName, lovValueFieldObject );
                        }
                    } catch ( IllegalArgumentException e ) {
                        String msg = "Unknown type " + lovValueFieldType + " for Lov " + lovRequestID;

                        logFatal( msg );

                        throw new EJBException( msg );
                    }
                }

                _lov.put( lovValue.getPK(), lovValue );
                _sortedLov.add( lovValue );
            }
        } catch ( SQLException Se ) {
            logError( "SQL Exception... ", Se );

            handleSQLException( Se );
        } catch ( NullPointerException NPe ) {
            String msg = "The first LOV_DEFINITION_ORDER in table LOV_DEFINITION must be 2... Check lov "
                         + lovRequestID;

            logFatal( msg );

            throw new EJBException( msg, NPe );
        } finally {
            // We close the various opened connections
            try {
                if ( resultSetCache != null ) {
                    resultSetCache.close();
                }

                if ( resultSetMetaData != null ) {
                    resultSetMetaData.close();
                }

                if ( resultSet != null ) {
                    resultSet.close();
                }
            } catch ( Exception Se ) {
                // Nothing else to do....
            }

            try {
                if ( preparedStatement != null ) {
                    preparedStatement.close();
                }
            } catch ( Exception Se ) {
                // Nothing else to do....
            }

            try {
                if ( statement != null ) {
                    statement.close();
                }
            } catch ( Exception Se ) {
                // Nothing else to do....
            }

            try {
                if ( connection != null ) {
                    connection.close();
                }
            } catch ( Exception Se ) {
                // Nothing else to do....
            }
        }

        logGraphEnd( "ejbLoad " + lovRequestID.toString() );
    }

    /**
     * Checks if the the Lov exist in the S_LOV_REQUESTS table. Used to
     * test for the LOV existence.
     *
     * @return The list of values having the given OID.
     *
     * @throws ObjectNotFoundException if the corresponding LOV doesn't exist.
     * @throws IllegalArgumentException if the given OID is null.
     */
    public Long ejbFindByPrimaryKey( Long p_OID ) throws ObjectNotFoundException {
        if ( p_OID == null ) {
            throw new IllegalArgumentException();
        }

        PreparedStatement preparedStatement = null;
        ResultSet resultSet = null;
        Connection connection = null;

        logGraphBegin( "ejbFindByPrimaryKey " + p_OID );

        // We only have to do something if the LOV isn't already in cache
        if ( !LovDefinitionCache.getInstance().isOIDInCache( p_OID ) ) {
            try {
                connection = getConnection( JNDI_DATABASE_NAME );
                preparedStatement = connection.prepareStatement( NAME_AND_REQUEST_FOR_OID_STATEMENT );
                preparedStatement.setString( 1, p_OID.toString() );
                resultSet = preparedStatement.executeQuery();

                // We should have somethig otherwise the LOV doesn't exist...
                if ( resultSet.next() == false ) {
                    String msg = "The LOV " + p_OID + " does not exist...";

                    logError( msg );

                    throw new ObjectNotFoundException( msg );
                }
            } catch ( SQLException Se ) {
                handleSQLException( Se );
            } finally {
                try {
                    // We close the various connection, resultset...
                    if ( resultSet != null ) {
                        resultSet.close();
                    }

                    if ( preparedStatement != null ) {
                        preparedStatement.close();
                    }

                    if ( connection != null ) {
                        connection.close();
                    }
                } catch ( SQLException Se ) {
                    handleSQLException( Se );
                }
            }
        }

        logGraphEnd( "ejbFindByPrimaryKey " + p_OID.toString() );

        // If we reach this place that means we found the expected LOV, returning its OID
        return p_OID;
    }

    /**
     * Finds the LOVs that are working on a given physical table. In other words
     * it enables us to get a Collection of LOVs that are related to a given DB
     * Table.<p>
     *
     * Is useful, for example, if the data in the DB are modified (a country is
     * added or removed) and we want to update the LOVs working on those data
     * (on the COUNTRIES table) for them to reflect the changes. First we need
     * to know what LOVs are working on those data and that's what this finder
     * retrieve for us.<p>
     *
     * You can check the Class description to have more informations about the
     * DB structure.
     *
     * @param p_tableName The name of the DB table that we want to match.
     *
     * @return The OIDs of the LOVs that refer to the given table name in their
     * TABLE_NAME field (cf. TABLE description).
     *
     * @throws IllegalArgumentException if the given table name is null.
     * @throws ObjectNotFoundException if none is found matching the given
     * table name.
     */
    public Collection ejbFindByPhysicalTable( String p_tableName ) throws FinderException {
        if ( p_tableName == null ) {
            throw new IllegalArgumentException();
        }

        Vector lovs = new Vector();
        PreparedStatement preparedStatement = null;
        ResultSet resultSet = null;
        Connection connection = null;

        logGraphBegin( "ejbFindByPhysicalTable " + p_tableName );

        try {
            connection = getConnection( JNDI_DATABASE_NAME );

            preparedStatement = connection.prepareStatement( LOV_OID_FOR_TABLE_NAME_STATEMENT );
            preparedStatement.setString( 1, p_tableName );
            resultSet = preparedStatement.executeQuery();

            while ( resultSet.next() ) {
                // We add each found OID to the collection to be returned
                Long lovOID = new Long( resultSet.getLong( 1 ) );
                lovs.add( lovOID );
            }

            if ( lovs.size() == 0 ) {
                String message = "No LOVs refering to the table name " + p_tableName;

                throw new ObjectNotFoundException( message );
            }
        } catch ( SQLException Se ) {
            handleSQLException( Se );
        } finally {
            try {
                // We close the various connection, resultset...
                if ( resultSet != null ) {
                    resultSet.close();
                }

                if ( preparedStatement != null ) {
                    preparedStatement.close();
                }

                if ( connection != null ) {
                    connection.close();
                }
            } catch ( SQLException Se ) {
                handleSQLException( Se );
            }
        }

        return lovs;
    }

    // *************************************************************************
    // ******************** LovBusinessInterface methods ***********************
    // *************************************************************************

    /**
     * Returns the LovValue corresponding to the given OID in the List Of Values
     * held in the implementing Object.
     *
     * @param p_lovValueOID The OID, in the List Of Values held in the implementing
     * Object, of the LovValue desired.
     *
     * @return the LovValue corresponding to the given oid or null if the
     * LovValue isn't present.
     *
     * @throws NoSuchLovValueException if the requested LovValue isn't found in
     * the LOV.
     *
     * @ejb.interface-method
     */
    public LovValue getLovValue( Long p_lovValueOID ) throws NoSuchLovValueException {
        Long lovOID = (Long) ejbContext.getPrimaryKey();

        logDebug( "getLovValue " + p_lovValueOID.toString() + " in LOV " + lovOID );

        if ( !_lov.containsKey( p_lovValueOID ) ) {
            throw new NoSuchLovValueException( lovOID, p_lovValueOID );
        }

        return (LovValue) _lov.get( p_lovValueOID );
    }

    /**
     * Gets the LOV mapped by OID.
     *
     * @return a HashMap of (key=OID, object=LovValue).
     *
     * @ejb.interface-method
     */
    public HashMap getLovValuesPerOID() {
        logDebug( "getLovValues " + ejbContext.getPrimaryKey().toString() );

        return _lov;
    }

    /**
     * Gets the LovValues in the order of their retrieval (for example if we
     * retrieve them by using a EJB BMP and the LOV_REQUESTS table we will get
     * them in the order of the result of the executed SQL).
     *
     * @return A list containing the LovValues.
     *
     * @ejb.interface-method
     */
    public List getListOfValues() {
        logDebug( "getSortedLovValues for the LOV " + ejbContext.getPrimaryKey().toString() );

        return _sortedLov;
    }

    // *************************************************************************
    // ***************************** Accessors *********************************
    // *************************************************************************

    /**
     * @return The LovMetaDataItems in a List.
     *
     * @ejb.interface-method
     */
    public List getMetaData() {
        logDebug( "getMetaData for the LOV " + ejbContext.getPrimaryKey().toString() );

        return _metaData;
    }
}
