/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */
package org.bsf.listOfValues;

import org.bsf.listOfValues.exceptions.NoSuchLovException;
import org.bsf.listOfValues.exceptions.NoSuchLovValueException;
import org.bsf.listOfValues.lovValue.LovValue;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import java.sql.*;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * This stateless ejb provides access to the list of values features. It
 * provides a way to access and update a LOV.
 *
 * @see LovBean
 * @see LovValue
 * @see LovBusinessInterface
 *
 *  @ejb:bean       type="Stateless"
 *                  name="LOVService"
 *                  jndi-name="ejb/LOVService"
 *                  local-jndi-name="ejb/LOVServiceLocal"
 *                  view-type="both"
 *
 *  @ejb:home       extends="javax.ejb.EJBHome"
 *                  local-extends="javax.ejb.EJBLocalHome"
 *
 *  @ejb:interface  extends="javax.ejb.EJBObject"
 *                  local-extends="javax.ejb.EJBLocalObject"
 *
 *  @ejb:ejb-ref    ejb-name="LOV"
 *                  view-type="local"
 *
 *  @ejb:transaction type="Required"
 *
 *  @ejb:resource-ref   res-name="jdbc/DBPool"
 *                      res-type="javax.sql.DataSource"
 *                      res-auth="Container"
 *
 *  @jboss:resource-ref res-ref-name="jdbc/DBPool"
 *                      resource-name="FwkDataSourceManager"
 *
 *  @jonas.bean
 *                      ejb-name="LOVService"
 *                      jndi-name="ejb/LOVService"
 *
 *  @jonas.resource
 *                       res-ref-name="jdbc/DBPool"
 *                       jndi-name="jdbc_1"
 *
 */
public class LovServiceBean extends org.bsf.commons.ejb.SessionAdapterBean {
    private static final String TABLE_NAME_FROM_LOV_OID_STATEMENT
            = "SELECT TABLE_NAME FROM BSF_LOV WHERE OID = ?";

    private LOVLocalHome _lovEntityLocalHome = null;

    private final String JNDI_DATABASE_NAME = "DBPool";

    /**
     * Provides access to the metadata for a given list of values.
     *
     * @param p_lovOID The OID of the list of values for which we want the
     * metadata.
     *
     * @return A list of  LovMetaDataItems for the LOV corresponding to the
     * given lovOID.
     *
     * @throws IllegalArgumentException if the given oid is null.
     * @throws FinderException if a lov with the given oid isn't found.
     *
     * @see LovBean
     * @see LovValue
     * @see LovMetaDataItem
     *
     * @ejb:interface-method
     */
    public List getLOVMetaData( Long p_lovOID ) throws FinderException {
        if ( p_lovOID == null ) {
            throw new IllegalArgumentException( "Need a non null oid to retrun anything..." );
        }

        List metaDatas = null;

        logGraphBegin( "getLOVMetaData for LOV " + p_lovOID );

        // We retrieve the LOV, a FinderExecption is thrown if none found
        LOVLocal lovLocal = _lovEntityLocalHome.findByPrimaryKey( p_lovOID );

        // We retrieve the metadata from the found Lov
        metaDatas = lovLocal.getMetaData();

        return metaDatas;
    }

    /**
     * Returns the LovValue having the oid p_lovValueOID in the LOV whose oid
     * is p_lovOID. If the LOV doesn't exist a NoSuchLovException is thrown
     * whereas a NoSuchLovValueException is thrown if, while existing, the
     * LOV does not contain the desired LovValue.
     *
     * @param p_lovOID The oid of the LOV that we want to parse.
     * @param p_lovValueOID The oid that we want to find in the selected LOV.
     *
     * @return The LovValue having the oid p_lovValueOID in the LOV whose oid is
     * p_lovOID.
     *
     * @throws NoSuchLovException if no LOV is found for the given p_lovOID.
     * @throws NoSuchLovValueException if no LovValue is found having the
     * p_lovValueOID in the selected LOV.
     * @throws IllegalArgumentException if the given p_lovOID or p_lovValueOID
     * is (are) null.
     *
     * @see LovValue
     *
     * @ejb:interface-method
     */
    public LovValue getLovValue( Long p_lovOID, Long p_lovValueOID )
            throws NoSuchLovException, NoSuchLovValueException {

        LOVLocal lovLocal = null;
        LovValue lovValue = null;

        if ( ( p_lovOID == null ) || ( p_lovValueOID == null ) ) {
            String msg = "Needs a non null lovOID (" + p_lovOID
                    + ") and p_lovValueOID (" + p_lovValueOID
                    + ") to return a LovValue...";

            throw new IllegalArgumentException( msg );
        }

        logDebug( "getLovValue " + p_lovValueOID + " in LOV " + p_lovOID );

        // A NoSuchLovException can be raised here...
        lovLocal = getLov( p_lovOID );

        // A NoSuchLovValueException can be raised here...
        lovValue = lovLocal.getLovValue( p_lovValueOID );

        return lovValue;
    }

    /**
     * Returns, in the order of the executed SQL, the list of LovValues
     * corresponding to the given p_lovOID.
     *
     * @param p_lovOID The oid of the LOV that we want to retrieve.
     *
     * @return A list of LovValues corresponding to the execution of the
     * given lovOID. The LovValues are returned as they are retrieve from
     * the execution of the SQL statement (cf. ORDER BY clause).
     *
     * @throws IllegalArgumentException if the given lovOID is null.
     * @throws NoSuchLovException if no LOV is having the given oid.
     *
     * @see LovBean
     * @see LovValue
     *
     * @ejb:interface-method
     */
    public List getListOfValues( Long p_lovOID ) throws NoSuchLovException {
        if ( p_lovOID == null ) {
            throw new IllegalArgumentException( "Need a non null oid to retrieve a LOV..." );
        }

        List listOfValues = null;
        LOVLocal lovLocal = null;

        logGraphBegin( "getSortedLOV for the lovOID " + p_lovOID );

        // NoSuchLovException can be thrown from here
        lovLocal = getLov( p_lovOID );

        // RemoteException can be thrown from here
        listOfValues = lovLocal.getListOfValues();

        logGraphEnd( "getSortedLOV for the lovOID " + p_lovOID );

        return listOfValues;
    }

    /**
     * Returns a map (LovValueOID, LovValue) of LovValues corresponding to the
     * given p_lovOID.
     *
     * @param p_lovOID The oid of the LOV that we want to retrieve.
     *
     * @return A list of LovValues corresponding to the execution of the
     * given p_lovOID. The LovValues are returned as they are retrieve from
     * the execution of the SQL statement (cf. ORDER BY clause).
     *
     * @throws IllegalArgumentException if the given p_lovOID is null.
     * @throws NoSuchLovException if no LOV is having the given oid.
     *
     * @see LovBean
     * @see LovValue
     *
     * @ejb:interface-method
     */
    public Map getLovValues( Long p_lovOID ) throws NoSuchLovException {
        LOVLocal lovLocal = null;
        Map mapLov = null;

        logGraphBegin( "getLovValues as a map for lovOID " + p_lovOID );

        lovLocal = getLov( p_lovOID );
        mapLov = lovLocal.getLovValuesPerOID();

        logGraphEnd( "getLovValues as a map for lovOID " + p_lovOID );

        return mapLov;
    }

    /**
     * Ensures that the LOVs using the same TABLE_NAME as the LOV identified
     * by the given OID are reloaded (in general due to a data update). It
     * includes, of course, the lov identified by the given lovOID.
     *
     * @param p_lovOID The oid of the LOV whose TABLE_NAME will be used to
     * refresh the LOVs.
     *
     * @throws IllegalArgumentException if the given p_lovOID is null.
     * @throws NoSuchLovException if there is no LOV having the given lovOID.
     *
     * @see LovBean
     * @see #resynchronizeListOfValues(String p_tableName)
     *
     * @ejb:interface-method
     */
    public void resynchronizeListOfValues( Long p_lovOID ) throws NoSuchLovException {
        if ( p_lovOID == null ) {
            throw new IllegalArgumentException( "Can't resynchronize for a null lovOID" );
        }

        String physicalTable = null;
        Connection connection = null;
        PreparedStatement preparedStatement = null;
        ResultSet rsTableName = null;

        logGraphBegin( "resynchronizeLOV " + p_lovOID );

        try {
            // Getting the physical table name
            connection = getConnection( JNDI_DATABASE_NAME );
            preparedStatement = connection.prepareStatement( TABLE_NAME_FROM_LOV_OID_STATEMENT );
            preparedStatement.setString( 1, p_lovOID.toString() );
            rsTableName = preparedStatement.executeQuery();

            // We should have a table name if the lov exist
            if ( rsTableName.next() ) {
                physicalTable = rsTableName.getString( 1 );
            } else {
                throw new NoSuchLovException( p_lovOID );
            }
        } catch( SQLException Se ) {
            handleSQLException( Se );
        } finally {
            try {
                // We close the opened jdbc objects
                if ( rsTableName != null ) {
                    rsTableName.close();
                }

                if ( preparedStatement != null ) {
                    preparedStatement.close();
                }

                if ( connection != null ) {
                    connection.close();
                }
            } catch( SQLException Se ) {
                logFatal( "Exception in finaly " + Se );
            }
        }

        // If we reach here we should have something to resynchronize
        resynchronizeListOfValues( physicalTable );
    }

    /**
     * Ensures that the LOVs referencing the TABLE_NAME are reseted. Usually
     * because the underlying data have changed.
     *
     * @param p_tableName The oid of the LOV whose TABLE_NAME will be used to
     * refresh the LOVs. Does nothing if the name is invalid (null, no match).
     *
     * @see LovBean
     * @see #resynchronizeListOfValues(Long p_lovOID)
     *
     * @ejb:interface-method
     */
    public void resynchronizeListOfValues( String p_tableName ) {
        if ( p_tableName == null ) {
            // Nothing to do...
            return;
        }

        try {
            Collection lovs = _lovEntityLocalHome.findByPhysicalTable( p_tableName );

            Iterator lovIterator = lovs.iterator();

            while ( lovIterator.hasNext() ) {
                Object aRef = lovIterator.next();

                // We need to narrow since it comes from thru collection from a finder
                LOVLocal aLovLocal = (LOVLocal) PortableRemoteObject.narrow( aRef, LOVLocal.class );

                // We remove it (ie. we clear the cache)
                aLovLocal.remove();
            }
        } catch( RemoveException removeException ) {
            handleExceptionAsSystemException( removeException );
        } catch( FinderException finderException ) {
            handleFinderException( finderException );
        }
    }

    /**
     * Utility method to provide access to the desired LOV.
     *
     * @param p_lovOID The oid of the desired LOV.
     *
     * @return A LovLocal corresponding to the desired LOV.
     *
     * @throws NoSuchLovException if no LOV is found having the given oid.
     */
    private LOVLocal getLov( Long p_lovOID ) throws NoSuchLovException {
        LOVLocal lovLocal = null;

        try {
            lovLocal = _lovEntityLocalHome.findByPrimaryKey( p_lovOID );
        } catch( ObjectNotFoundException ONFe ) {
            throw new NoSuchLovException( p_lovOID );
        } catch( FinderException Fe ) {
            handleFinderException( Fe );
        }

        return lovLocal;
    }

    // *************************************************************************
    // ******************** SesssionAdapterBean Methods ************************
    // *************************************************************************

    public void ejbCreate() throws CreateException {
        logDebug( "ejbCreate() -> Retrieving the home(s)..." );

        // Retrieves the local home of the Lov Entity bean and keeps it for further use
        try {
            InitialContext ic = new InitialContext();
            _lovEntityLocalHome = (LOVLocalHome) ic.lookup( "java:comp/env/ejb/LOV" );
        } catch( NamingException e ) {
            handleExceptionAsSystemException( e );
        }
    }
}
