package org.bsf.listOfValues.client;

import org.bsf.listOfValues.LOVService;
import org.bsf.listOfValues.exceptions.NoSuchLovException;
import org.bsf.listOfValues.exceptions.NoSuchLovValueException;
import org.bsf.listOfValues.lovValue.LovValue;

import java.rmi.RemoteException;
import java.util.*;

/**
 * Enables access to the LOV using the given LovService. Is usaully used as a
 * cache between the server and the client.
 *
 * @see LovValue
 * @see LOVService
 * @see LovListener
 */
public class LovManager {
    /**
     * To specify that a listener is interested in all the lovs. Remember that
     * a Lov OID shouldn't be negative (it's a rule :o).
     */
    public static final Long ALL_LOVS = new Long( -123321 );

    private static LOVService _lovService = null;
    private static final int DEFAULT_HASH_TABLE_SIZE = 30;

    /**
     * Holds the retrieved list of values (aLovOID, aList of LovValue).
     */
    private static Hashtable _listOfValues = new Hashtable( DEFAULT_HASH_TABLE_SIZE );

    /**
     * Holds the retrieved list of values (aLovOID, (aLovValuePK, aList of LovValue)).
     */
    private static Hashtable _listOfValuesPerPK = new Hashtable( DEFAULT_HASH_TABLE_SIZE );

    /**
     * Holds the listeners (LovOID, List of LovListeners).
     */
    private static Hashtable _lovListeners = new Hashtable( DEFAULT_HASH_TABLE_SIZE );

    // *************************************************************************
    // ************************* Static Accessors ******************************
    // *************************************************************************

    /**
     * @return The lov service defined to be used. Can be null if not set.
     *
     * @see #setLovService
     */
    protected static LOVService getLovService() {
        return _lovService;
    }

    /**
     * Set the LovService to be used to access the LovServiceBean.
     *
     * @param p_lovService The LovService to use.
     *
     * @throws java.lang.IllegalArgumentException if the given LovService is null.
     */
    public static void setLovService( LOVService p_lovService ) {
        _lovService = p_lovService;
    }

    // *************************************************************************
    // ***************************** Static Methods ****************************
    // *************************************************************************

    /**
     * Return the requested list of values, checking in its cache first. If not
     * found, request it from the actually registered LovService, and store it
     * in the cache before returning it.
     *
     * @param p_lovOID The OID of the desired list of values.
     * @return The list of values corresponding to the given LovOID.
     *
     * @throws org.bsf.listOfValues.exceptions.NoSuchLovException If the LOV
     * doesn't exist.
     * @throws java.rmi.RemoteException If a communication problem occurs.
     * @throws java.lang.IllegalArgumentException if the given LovOID is null.
     * @throws java.lang.IllegalStateException if no LovService is registered
     */
    public static List getListOfValues( Long p_lovOID ) throws NoSuchLovException, RemoteException {
        if ( getLovService() == null ) {
            throw new IllegalStateException( "No LovService has been set..." );
        }

        if ( p_lovOID == null ) {
            throw new IllegalArgumentException( "Can't return a LOV with a null lovOID..." );
        }

        // Do we have it in cache ?
        List listOfValues = (List) _listOfValues.get( p_lovOID );

        // Do we need to make a server call ?
        if ( listOfValues == null ) {
            // We retrieve it from the server
            listOfValues = _lovService.getListOfValues( p_lovOID );

            // We add it to the cache for the next call
            _listOfValues.put( p_lovOID, listOfValues );
        }

        return listOfValues;
    }

    /**
     * Return the requested list of values, checking in its cache first. If not
     * found, request it from the actually registered LovService, and store it
     * in the cache before returning it. It register the given LovListener
     * as interested in notification of LOV modification.
     *
     * @param p_lovListener The lovListener to be notified if this requested
     * lov changes. Can be null (no effect on the listener list).
     * @param p_lovOID The OID of the desired list of values.
     * @return The list of values corresponding to the given LovOID.
     *
     * @throws org.bsf.listOfValues.exceptions.NoSuchLovException If the LOV
     * doesn't exist.
     * @throws java.rmi.RemoteException If a communication problem occurs.
     * @throws java.lang.IllegalArgumentException if the given LovOID is null.
     * @throws java.lang.IllegalStateException if no LovService is registered
     */
    public static List getListOfValues( LovListener p_lovListener, Long p_lovOID )
            throws NoSuchLovException, RemoteException {

        List listOfValues = getListOfValues( p_lovOID );

        // We add the given listener to the list of listeners
        addLovManagerListener( p_lovListener, p_lovOID );

        return listOfValues;
    }

    /**
     * Return the requested LovValue that belongs to the specified LOV. The whole
     * list of values is retrieved from the server if not yet in cache. It creates
     * a map to store the LovValue per their oid to accelerate later data access.
     * Can be really useful for example when the getLovValue method is used in an
     * often called renderer (TableCellRenderer for example).
     *
     * @param p_lovOID The OID of the desired list of values.
     * @param p_lovValuePK The PK of the LovValue (belonging to the LOV
     * whose oid is p_lovOID) desired.
     * @return The list of values corresponding to the given LovOID.
     *
     * @throws org.bsf.listOfValues.exceptions.NoSuchLovValueException if the
     * requested lov value doesn't exist.
     * @throws org.bsf.listOfValues.exceptions.NoSuchLovException if the LOV
     * doesn't exist.
     * @throws java.rmi.RemoteException if a communication problem occurs.
     * @throws java.lang.IllegalStateException if no LovService is registered.
     * @throws java.lang.ClassCastException if the LOV contains anything else
     * than LovValue.
     */
    public static LovValue getLovValue( Long p_lovOID, Object p_lovValuePK )
            throws NoSuchLovValueException, NoSuchLovException, RemoteException {

        if ( getLovService() == null ) {
            throw new IllegalStateException( "No LovService has been set..." );
        }

        if ( p_lovOID == null || p_lovValuePK == null ) {
            throw new IllegalArgumentException( "Need a non null lovOID and lovValueOID..." );
        }

        // Is it already in cache ?
        Hashtable lovValuePerPK = (Hashtable) _listOfValuesPerPK.get( p_lovOID );

        // Do we need to retrieve the info from the server ?
        if ( lovValuePerPK == null ) {
            List listOfValues = getListOfValues( p_lovOID );

            Iterator iterator = listOfValues.iterator();
            lovValuePerPK = new Hashtable( listOfValues.size() );

            // We add each LovValue to the map (lovValueOID, lovValue)
            while ( iterator.hasNext() ) {
                LovValue lovValue = (LovValue) iterator.next();
                lovValuePerPK.put( lovValue.getPK(), lovValue );
            }

            _listOfValuesPerPK.put( p_lovOID, lovValuePerPK );
        }

        // We retrieve the requested LovValue from the hashTable
        LovValue lovValue = (LovValue) lovValuePerPK.get( p_lovValuePK);

        if ( lovValue == null ) {
            throw new NoSuchLovValueException( "No LovValue " + p_lovValuePK + " in Lov " + p_lovOID );
        }

        return lovValue;
    }

    /**
     * Return the requested LovValue that belongs to the specified LOV. The whole
     * list of values is retrieved from the server if not yet in cache. It creates
     * a map to store the LovValue per their oid to accelerate later data access.
     * Can be really useful for example when the getLovValue method is used in an
     * often called renderer (TableCellRenderer for example).
     *
     * @param p_lovListener The listener to be notify is the requested LOV changes.
     * Can be null (no effect on the listeners list).
     * @param p_lovOID The OID of the desired list of values.
     * @param p_lovValuePK The OID of the LovValue (belonging to the LOV
     * whose oid is p_lovOID) desired.
     * @return The list of values corresponding to the given LovOID.
     *
     * @throws org.bsf.listOfValues.exceptions.NoSuchLovValueException if the
     * requested lov value doesn't exist.
     * @throws org.bsf.listOfValues.exceptions.NoSuchLovException if the LOV
     * doesn't exist.
     * @throws java.rmi.RemoteException if a communication problem occurs.
     * @throws java.lang.IllegalStateException if no LovService is registered.
     * @throws java.lang.ClassCastException if the LOV contains anything else
     * than LovValue.
     */
    public static LovValue getLovValue( LovListener p_lovListener, Long p_lovOID, Object p_lovValuePK )
            throws NoSuchLovValueException, NoSuchLovException, RemoteException {

        LovValue lovValue = getLovValue( p_lovOID, p_lovValuePK );

        // We add the listener to the list of listeners
        addLovManagerListener( p_lovListener, p_lovOID );

        return lovValue;
    }

    /**
     * Clears the server and client cache and notifies the registered listeners that the
     * LOV has changed. The LOV isn't reloaded through this call... It will notifies
     * the listeners (if any). It will be reloaded the next time this lov is loaded.
     *
     * @param p_lovOID The OID of the Lov to synchronize with the server.
     *
     * @throws NoSuchLovException If no lov has the given OID.
     * @throws RemoteException If a communication problem occurs.
     * @throws IllegalArgumentException if the given oid is null.
     * @throws IllegalStateException if no LovService has been set.
     *
     * @see LOVService
     * @see #setLovService
     */
    public static void resynchronizeLov( Long p_lovOID ) throws NoSuchLovException, RemoteException {
        if ( getLovService() == null ) {
            throw new IllegalStateException( "No LovService has been set..." );
        }

        if ( p_lovOID == null ) {
            throw new IllegalArgumentException( "Need a non null lovOID to refresh a Lov..." );
        }

        // We resynchronize the Lov
        getLovService().resynchronizeListOfValues( p_lovOID );

        // We clear the local cache
        clearLocalCache( p_lovOID );

        // We notify the registered listeners (if any)
        notifyLovListeners( p_lovOID );
    }

    /**
     * Removes the LOV, whose OID is the given oid, from the cache.
     *
     * @param p_lovOID The OID of the LOV to remove from the cache. Should not
     * be null.
     *
     * @throws IllegalArgumentException if the given oid is null.
     */
    public static void clearLocalCache( Long p_lovOID ) {
        if ( p_lovOID == null ) {
            throw new IllegalArgumentException( "Need a non null lovOID to refresh a Lov..." );
        }

        // We clear the cache(s)
        _listOfValues.remove( p_lovOID );
        _listOfValuesPerPK.remove( p_lovOID );
    }

    // *************************************************************************
    // *************************** Listener Methods ****************************
    // *************************************************************************

    /**
     * Registers a LovListener for the given lovOID. If the lovOID equals
     * ALL_LISTENERS the LovListener will be notified of the modification
     * of every lov. Does nothing if the LovListener is null.
     *
     * @param p_lovListener The LovListener that we want to be notified.
     * @param p_lovOID The OID of the LOV that the LovListener listens to, if
     * equals to ALL_LOVS can be used to specify that a listener is interested
     * by all the Lovs.
     */
    public static void addLovManagerListener( LovListener p_lovListener, Long p_lovOID ) {
        if ( p_lovListener == null ) {
            return;
        }

        List listeners = (List) _lovListeners.get( p_lovOID );

        if ( listeners == null ) {
            listeners = new Vector( 10 );
            listeners.add( p_lovListener );

            _lovListeners.put( p_lovOID, listeners );
        } else {
            if ( !listeners.contains( p_lovListener ) ) {
                listeners.add( p_lovListener );
            }
        }
    }

    /**
     * Removes the given LovListener from the registered listeners.
     *
     * @param p_lovListener The LovListener that we want to remove.
     *
     * @throws IllegalArgumentException if the LovListener is null.
     */
    public static void removeLovManagerListener( LovListener p_lovListener ) {
        if ( p_lovListener == null ) {
            throw new IllegalArgumentException( "Can't remove a null listener..." );
        }

        Enumeration lovOIDs = _lovListeners.elements();

        while ( lovOIDs.hasMoreElements() ) {
            // We remove the listener from all the lists
            ( (List) lovOIDs.nextElement() ).remove( p_lovListener );
        }
    }

    /**
     * Notify all the listeners interested for the given lovOID.
     *
     * @param p_lovOID The OID of the LOV of which we want to notify the
     * modificaiton.
     *
     * @throws IllegalArgumentException if the given lovOID is null.
     */
    public static void notifyLovListeners( Long p_lovOID ) {
        if ( p_lovOID == null ) {
            throw new IllegalArgumentException( "Can't notify for a null lovOID..." );
        }

        // First we take care of the listeners that listen to ALL_LOVS
        Vector listenersList = new Vector();
        Collection listeners = ( (Collection) _lovListeners.get( ALL_LOVS ) );

        if ( listeners != null ) {
            listenersList.addAll( listeners );
        }

        // And we add the one not present yet but that listen to the specified lov
        listeners = (List) _lovListeners.get( p_lovOID );

        if ( listeners != null ) {
            Iterator iterator = listeners.iterator();

            while ( iterator.hasNext() ) {
                Object object = iterator.next();

                if ( !listenersList.contains( object ) ) {
                    listenersList.add( object );
                }
            }
        }

        // And finally we notify each and every one of them
        Iterator listenersIterator = listenersList.iterator();

        while ( listenersIterator.hasNext() ) {
            ( (LovListener) listenersIterator.next() ).lovHasChanged( p_lovOID );
        }
    }
}