/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */

package org.bsf.listOfValues.lovValue;

import java.util.HashMap;
import java.util.Iterator;

/**
 * A default implementation of the LovValue interface. It represents a row of
 * a list of values.
 *
 * @see LovValue
 */
public class DefaultLovValue implements LovValue {
    public static final String FIELD_SEPARATOR = " , ";

    protected Object _pk;
    protected HashMap _fields;

    // *************************************************************************
    // *************************** Constructor *********************************
    // *************************************************************************

    public DefaultLovValue( Object p_pk ) {
        _pk = p_pk;
        _fields = new HashMap();
    }

    // *************************************************************************
    // ************************** LovValue methods *****************************
    // *************************************************************************

    /**
     * This method is deprecated but should keep working as long as the first
     * column returned is a Long.
     *
     * @return The oid of this particular LovValue.
     *
     * @see LovValue
     * @see #getPK()
     *
     * @deprecated should use getPK()
     */
    public Long getOid() {
        return (Long) _pk;
    }

    /**
     * This method should be use instead of the old getOid() method.
     *
     * @return The pk of this particular LovValue.
     *
     * @see LovValue
     * @see org.bsf.listOfValues.LovBean
     */
    public Object getPK() {
        return _pk;
    }

    /**
     * @param p_fieldName  The name of the field for which the value is desired.
     *
     * @return the value associated with the given field name or null if no
     * match is found.
     */
    public Object getFieldValue( String p_fieldName ) {
        if ( !_fields.containsKey( p_fieldName ) ) {
            return null;
        }

        return _fields.get( p_fieldName );
    }

    /**
     * It will replace any previous value with the same field name.
     *
     * @param p_fieldName The field name identifying the given value.
     * @param p_fieldValue The value to be stored for the given field name.
     */
    public void addField( String p_fieldName, Object p_fieldValue ) {
        _fields.put( p_fieldName, p_fieldValue );
    }

    // *************************************************************************
    // ********************** Overriden Object methods *************************
    // *************************************************************************

    /**
     * @param p_lovValue The other LovValue to test against. The check is done
     * on the Class of the other LovValue (needs to be a DefaultLovValue to be
     * considered equal), the pk and on the fields (map comparison).
     *
     * @return true if two LovValue are equals, false otherwise.
     */
    public boolean equals( Object p_lovValue ) {
        if ( p_lovValue == this ) {
            return true;
        }

        if ( !( p_lovValue instanceof DefaultLovValue ) ) {
            return false;
        }

        DefaultLovValue otherLov = (DefaultLovValue) p_lovValue;

        if ( getPK().equals( otherLov.getPK() ) && _fields.equals( otherLov._fields ) ) {
            return true;
        }

        return false;
    }

    /**
     * @return a String representation of this LovValue, displaying the
     * LovValue PK then each field.
     *
     * ex:
     *  12, SHORT_NAME=FR, LONG_NAME=FRANCE, REGION=EUROPE, HEMISPHERE=NORTH HEMISPHERE
     *  13, SHORT_NAME=UK, LONG_NAME=UNITED KINGDOM, REGION=EUROPE, HEMISPHERE=NORTH HEMISPHERE
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        Iterator iter = _fields.entrySet().iterator();

        sb.append( _pk );

        while ( iter.hasNext() ) {
            sb.append( FIELD_SEPARATOR );

            Object object = iter.next();
            sb.append( object );
        }

        return sb.toString();
    }
}
