/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */

package org.bsf.listOfValues.util;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

/**
 * This class is used to store the "lov definitions" (in a LovEntityCacheItem)
 * to prevent the reexecution of a select statement to obtain the to be executed
 * SQL statement defined in the corresponding LOV.<p>
 *
 * Note that this Class is used as a singleton and that the LovEntityCacheItem
 * is never used outside of this cache.
 *
 * @see org.bsf.listOfValues.util.LovDefinitionCache.LovEntityCacheItem (in this file as well)
 */
public class LovDefinitionCache {
    // *************************************************************************
    // ********************* LovEntityCacheItem definition *********************
    // *************************************************************************

    /**
     * Stores the needed informations related to the LovBean (mainly the
     * SQL statement).
     */
    class LovEntityCacheItem {
        private List _types;
        private String _lovName;
        private String _lovRequest;

        public LovEntityCacheItem( String p_lovName, String p_lovRequest, List p_metaData ) {
            _types = p_metaData;
            _lovName = p_lovName;
            _lovRequest = p_lovRequest;
        }

        public String getRequest() {
            return _lovRequest;
        }

        public String getLovName() {
            return _lovName;
        }

        public List getTypes() {
            return _types;
        }
    }

    // *************************************************************************
    // ********************* Static part (field && method) *********************
    // *************************************************************************

    private static LovDefinitionCache _lovCacheInstance = null;

    /**
     * Gives access to the singleton.
     *
     * @return the singleton
     */
    public static LovDefinitionCache getInstance() {
        if ( _lovCacheInstance == null ) {
            synchronized( LovDefinitionCache.class ) {
                if ( _lovCacheInstance == null )
                    _lovCacheInstance = new LovDefinitionCache();
            }
        }

        return _lovCacheInstance;
    }

    // *************************************************************************
    // *********************** LovDefinitionCache definition *****************************
    // *************************************************************************

    /**
     * Cache that holds the LovEntityCacheItems. Informations are stored as
     * (aLovOID, aLovEntityCacheItem). Initial cache size is 50 and shuold
     * be enough for most applications (otherwise the map extends itself,
     * minor performance cost).
     */
    private HashMap _cache = new HashMap( 50 );

    private LovDefinitionCache() {
        // We don't want this Class to be instantiated from outside
    }

    /**
     * Returns the sql statement (or null if no match is found) stored in the
     * LovDefinitionCache and corresponding to the Lov with the given OID.
     *
     * @param p_lovOID The OID of the LOV whose SQL statement is desired.
     * @return the sql statement stored in the LovDefinitionCache corresponding to
     * the given OID or null if the given oid is not found.
     *
     * @see #isOIDInCache
     */
    public String getRequestForLovOID( Long p_lovOID ) {
        String request = null;
        LovEntityCacheItem item = (LovEntityCacheItem) _cache.get( p_lovOID );

        if ( item != null ) {
            // We have a request (ie. SQL statement), we will return it
            request = item.getRequest();
        }

        return request;
    }

    /**
     * Returns a collection of type corresponding to the the metadata(or null if
     * no match is found) stored in the LovDefinitionCache and corresponding to the Lov
     * with the given OID.
     *
     * @param p_lovOID The OID of the LOV whose types are desired.
     * @return the List of types stored in the LovDefinitionCache corresponding to
     * the given OID or null if the given oid is not found.
     *
     * @see #isOIDInCache
     */
    public List getTypesForLovOID( Long p_lovOID ) {
        List types = null;
        LovEntityCacheItem item = (LovEntityCacheItem) _cache.get( p_lovOID );

        if ( item != null ) {
            // We have a types (ie. SQL statement), we will return it
            types = item.getTypes();
        }

        return types;
    }

    /**
     * Returns whether or not informations, for a given LOV, are stored in the
     * LovDefinitionCache.
     *
     * @param  p_lovOID  The OID of the LOV for which we want to know if it is
     * in cache.
     * @return true if we have the informations for the given LovOID in the
     * cache and false otherwise.
     */
    public boolean isOIDInCache( Long p_lovOID ) {
        return _cache.containsKey( p_lovOID );
    }

    /**
     * Returns whether or not informations, for a given LOV, are stored in the
     * LovDefinitionCache.
     *
     * @param  p_lovName The name of the LOV for which we want to know if it is
     * in cache.
     * @return true if we have the informations for the given LovName in the
     * cache and false otherwise (or if the name is null).
     */
    public boolean isNameInCache( String p_lovName ) {
        boolean isNameInCache = true;

        if ( p_lovName == null || getLovOID( p_lovName ) == null ) {
            isNameInCache = false;
        }

        return isNameInCache;
    }

    /**
     * Adds a new item to the cache (will use a LovEntityCacheItem to do so).
     *
     * @param p_lovOID     The LovOID for which we want to store the informations.
     * @param p_lovName    The LovName corresponding to the LovOID.
     * @param p_lovRequest The LovRequest corresponding to the LovOID.
     *
     * @throws java.lang.IllegalArgumentException if the OID or the NAME or the REQUEST
     * is (are) null.
     */
    public void addToCache( Long p_lovOID, String p_lovName, String p_lovRequest, List p_metaData ) {
        if ( p_lovOID == null || p_lovName == null || p_lovRequest == null ) {
            String msg = "Need non null (LovOID,LovName,LovRequest) to store them in cache";

            throw new IllegalArgumentException( msg );
        }

        _cache.put( p_lovOID, new LovEntityCacheItem( p_lovName, p_lovRequest, p_metaData ) );
    }

    /**
     * Returns the LovOID corresponding to a given LovName or null if no match is
     * found in the cache. Note that it only checks in the cache and not anywhere
     * else.
     *
     * @param  p_lovName The name for which we want to obtain the LovOID.
     * @return The LovOID corresponding to the given LovName, null if no match
     * is found in the cache.
     */
    public Long getLovOID( String p_lovName ) {
        Iterator iter = _cache.keySet().iterator();

        while ( iter.hasNext() ) {
            Long OID = (Long) iter.next();

            LovEntityCacheItem item = (LovEntityCacheItem) _cache.get( OID );

            if ( item.getLovName().equals( p_lovName ) ) {
                return OID;
            }
        }

        return null;
    }
}
