/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */
package org.bsf.listOfValues.util;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * This class contains the definition of the type used by the LovBean and the
 * methods to handle the resultset regarding those types. Boolean are handled
 * from 0(false) and 1 (true). The type comparaisons are case insensitive and
 * check with Strings like "java.lang.Double" or "java.lang.String"...
 *
 */
public class LovTypeManager {
    // The available types
    public static final String BOOLEAN_TYPE = "java.lang.Boolean";
    public static final String DOUBLE_TYPE = "java.lang.Double";
    public static final String FLOAT_TYPE = "java.lang.Float";
    public static final String INTEGER_TYPE = "java.lang.Integer";
    public static final String LONG_TYPE = "java.lang.Long";
    public static final String STRING_TYPE = "java.lang.String";
    public static final String DATE_TYPE = "java.util.Date";
    public static final String SQL_DATE_TYPE = "java.sql.Date";

    /**
     * @param p_rs The resultset to use as source of the needed Object.
     * @param p_columnIndex The index of the column that we want to retrieve.
     *
     * @return The desired Object formatted as an Integer or null if the Object
     * in the resultset is null.
     *
     * @throws SQLException If a database access error occurs.
     */
    public static Integer getInteger( ResultSet p_rs, int p_columnIndex ) throws SQLException {
        if ( p_rs.getBigDecimal( p_columnIndex ) != null ) {
            return new Integer( p_rs.getInt( p_columnIndex ) );
        } else {
            return null;
        }
    }

    /**
     * @param p_rs The resultset to use as source of the needed Object.
     * @param p_columnIndex The index of the column that we want to retrieve.
     *
     * @return The desired Object formatted as a Long or null if the Object
     * in the resultset is null.
     *
     * @throws SQLException If a database access error occurs.
     */
    public static Long getLong( ResultSet p_rs, int p_columnIndex ) throws SQLException {
        if ( p_rs.getBigDecimal( p_columnIndex ) != null ) {
            return new Long( p_rs.getLong( p_columnIndex ) );
        } else {
            return null;
        }
    }

    /**
     * @param p_rs The resultset to use as source of the needed Object.
     * @param p_columnIndex The index of the column that we want to retrieve.
     *
     * @return The desired Object formatted as a String.
     *
     * @throws SQLException If a database access error occurs.
     */
    public static String getString( ResultSet p_rs, int p_columnIndex ) throws SQLException {
        return p_rs.getString( p_columnIndex );
    }

    /**
     * @param p_rs The resultset to use as source of the needed Object.
     * @param p_columnIndex The index of the column that we want to retrieve.
     *
     * @return The desired Object formatted as a Boolean (0 is handled as a
     * Boolean.FALSE while 1 is handled as a Boolean.TRUE) or null if the Object
     * in the resultset is null.
     *
     * @throws SQLException If a database access error occurs.
     */
    public static Boolean getBoolean( ResultSet p_rs, int p_columnIndex ) throws SQLException {
        if ( p_rs.getString( p_columnIndex ) != null ) {
            return p_rs.getString( p_columnIndex ).equals( "0" ) ? Boolean.FALSE : Boolean.TRUE;
        } else {
            return null;
        }
    }

    /**
     * @param p_rs The resultset to use as source of the needed Object.
     * @param p_columnIndex The index of the column that we want to retrieve.
     *
     * @return The desired Object formatted as a Double or null if the Object
     * in the resultset is null.
     *
     * @throws SQLException If a database access error occurs.
     */
    public static Double getDouble( ResultSet p_rs, int p_columnIndex ) throws SQLException {
        if ( p_rs.getBigDecimal( p_columnIndex ) != null ) {
            return new Double( p_rs.getDouble( p_columnIndex ) );
        } else {
            return null;
        }
    }

    /**
     * @param p_rs The resultset to use as source of the needed Object.
     * @param p_columnIndex The index of the column that we want to retrieve.
     *
     * @return The desired Object formatted as a Float or null if the Object
     * in the resultset is null.
     *
     * @throws SQLException If a database access error occurs.
     */
    public static Float getFloat( ResultSet p_rs, int p_columnIndex ) throws SQLException {
        if ( p_rs.getBigDecimal( p_columnIndex ) != null ) {
            return new Float( p_rs.getFloat( p_columnIndex ) );
        } else {
            return null;
        }
    }

    /**
     * @param p_rs The resultset to use as source of the needed Object.
     * @param p_columnIndex The index of the column that we want to retrieve.
     *
     * @return The desired Object formatted as a Date or null if the Object
     * in the resultset is null.
     *
     * @throws SQLException If a database access error occurs.
     */
    public static java.util.Date getDate( ResultSet p_rs, int p_columnIndex ) throws SQLException {
        if ( p_rs.getDate( p_columnIndex ) != null ) {
            return new java.util.Date( p_rs.getDate( p_columnIndex ).getTime() );
        } else {
            return null;
        }
    }

    /**
     * @param p_rs The resultset to use as source of the needed Object.
     * @param p_columnIndex The index of the column that we want to retrieve.
     *
     * @return The desired Object formatted as a SQL Date or null if the Object
     * in the resultset is null.
     *
     * @throws SQLException If a database access error occurs.
     */
    public static java.sql.Date getSqlDate( ResultSet p_rs, int p_columnIndex ) throws SQLException {
        if ( p_rs.getDate( p_columnIndex ) != null ) {
            return new java.sql.Date( p_rs.getDate( p_columnIndex ).getTime() );
        } else {
            return null;
        }
    }

    /**
     * @param p_type The type to use for the returned Object (should be a String
     * like "java.lang.Long" or "java.lang.String".
     * @param p_rs The resultset to use as source of the needed Object.
     * @param p_columnIndex The index of the column that we want to retrieve.
     *
     * @return The desired Object retrieve from the resultset as the given type or
     * null if the Object in the resultset is null.
     *
     * @throws SQLException If a database access error occurs.
     * @throws IllegalArgumentException if the type or the resultset is null or
     * if the type is unknown.
     */
    public static Object getObjectOfType( String p_type, ResultSet p_rs, int p_columnIndex ) throws SQLException {
        if ( p_type == null || p_rs == null ) {
            String msg = "Need a non null typeOID and resultset to return an Object...";
            throw new IllegalArgumentException( msg );
        }

        if ( INTEGER_TYPE.equalsIgnoreCase( p_type ) ) {
            return getInteger( p_rs, p_columnIndex );
        } else if ( LONG_TYPE.equalsIgnoreCase( p_type ) ) {
            return getLong( p_rs, p_columnIndex );
        } else if ( STRING_TYPE.equalsIgnoreCase( p_type ) ) {
            return getString( p_rs, p_columnIndex );
        } else if ( BOOLEAN_TYPE.equalsIgnoreCase( p_type ) ) {
            return getBoolean( p_rs, p_columnIndex );
        } else if ( DOUBLE_TYPE.equalsIgnoreCase( p_type ) ) {
            return getDouble( p_rs, p_columnIndex );
        } else if ( FLOAT_TYPE.equalsIgnoreCase( p_type ) ) {
            return getFloat( p_rs, p_columnIndex );
        } else if ( DATE_TYPE.equalsIgnoreCase( p_type ) ) {
            return getDate( p_rs, p_columnIndex );
        } else if ( SQL_DATE_TYPE.equalsIgnoreCase( p_type ) ) {
            return getSqlDate( p_rs, p_columnIndex );
        }

        throw new IllegalArgumentException( "Unknown type " + p_type );
    }
}