/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */

package org.bsf.listOfValues;

import junit.framework.TestCase;
import org.bsf.listOfValues.client.LovManager;
import org.bsf.listOfValues.exceptions.NoSuchLovException;
import org.bsf.listOfValues.exceptions.NoSuchLovValueException;
import org.bsf.remoting.EJBDefinition;
import org.bsf.remoting.http.HttpServiceFactory;

import java.util.List;

/**
 * The LOVs with OIDs 1 and 2 must exist in order for this test to succeed.
 * You can launch them on other lovs as well, just choose the OIDs to use in
 * FIRST_LOV_TO_TEST and SECOND_LOV_TO_TEST. Check the MySQL_LOV_EXAMPLES.sql
 * file to have some LOVs examples.
 */
public class TestLovService extends TestCase {
    private HttpServiceFactory factory = new HttpServiceFactory();

    // For the retrieval tests
    private static final Long FIRST_LOV_TO_TEST = new Long( 1 );
    private static final Long SECOND_LOV_TO_TEST = new Long( 3 );

    // For the Missing Lov test
    private static final Long NOT_A_LOV_OID = new Long( 9999 );
    private static final Long NOT_A_LOV_VALUE_OID_IN_FIRST_LOV_TO_TEST = new Long( 0 );

    EJBDefinition lovServiceDefinition = new EJBDefinition( "ejb/LOVService",
                                                            "org.bsf.listOfValues.LOVServiceHome",
                                                            "org.bsf.listOfValues.LOVService" );

    public TestLovService( String s ) {
        super( s );

        // The port to use, on the server, to communicate (default is 8080)
        factory.setPort( 8080 );

        // The location of the server on which the server is deployed
        factory.setHost( "localhost" );

        // The context of the remoting (in the ear def)
        factory.setServerContext( "testLov" );

        try {
            // We store the _lovService in the LovManager... It will handle the communication
            // and the caches on the client side (useless here :o) for us. Just to explain how
            // to use the LovManager along the LOVs.
            LovManager.setLovService( (LOVService) factory.getService( lovServiceDefinition ) );
        } catch( Exception e ) {
            System.out.println( "Unable to retrieve the LovService... Will exit..." );

            // We display the stack trace
            e.printStackTrace();

            // And exit
            System.exit( 0 );
        }
    }

    /**
     * To test that the LOV retrieval is working. If t is not working check the
     * FIRST and SECOND_LOV_TO_TEST values (should be the one of existing and
     * working LOVs...)
     */
    public void testLovRetrieval() throws Exception {
        try {
            System.out.println( "\nRunning: testLovRetrieval()" );

            // First LOV
            printLov( FIRST_LOV_TO_TEST, LovManager.getListOfValues( FIRST_LOV_TO_TEST ) );

            // Second LOV
            printLov( SECOND_LOV_TO_TEST, LovManager.getListOfValues( SECOND_LOV_TO_TEST ) );
        } catch( NoSuchLovException e ) {
            fail();
        }
    }

    /**
     * To test that a NoSuchLovException is thrown when requesting a non
     * existing LOV.
     */
    public void testMissingLov() throws Exception {
        try {
            System.out.println( "\nRunning: testMissingLov()" );

            LovManager.getListOfValues( NOT_A_LOV_OID );
        } catch( NoSuchLovException e ) {
            // Should be raised
            System.out.println( "\n[Success] testMissingLov: We managed to get the NoSuchLovException..." );

            // We stop here
            return;
        }

        // If we reach this point it is a faillure !
        fail();
    }

    /**
     * To test that a NoSuchLovValueException is thrown when requesting a non
     * existing LovValue.
     */
    public void testMissingLovValue() throws Exception {
        try {
            System.out.println( "\nRunning: testMissingLovValue()" );

            LovManager.getLovValue( FIRST_LOV_TO_TEST, NOT_A_LOV_VALUE_OID_IN_FIRST_LOV_TO_TEST );
        } catch( NoSuchLovException e ) {
            // The LOV should be present...
            fail();
        } catch( NoSuchLovValueException e ) {
            // Should be raised
            System.out.println( "\n[Success] testMissingLovValue: We managed to get the NoSuchLovValueException..." );

            // Success, we stop here !!!
            return;
        }

        // If we reach this point it is a faillure !
        fail();
    }

    private void printLov( Long p_lovOID, List p_lov ) {
        System.out.println( "\nDisplaying LOV: " + p_lovOID );

        if ( p_lov == null ) {
            System.out.println( "Null LOV..." );
        } else {
            for ( int index = 0 ; index < p_lov.size() ; index++ ) {
                System.out.println( ( p_lov.get( index ) ) );
            }
        }
    }
}